



python "%PREFIX%\\Library\\bin\\g-ir-scanner" --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
g-ir-compiler --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
for /f "usebackq tokens=*" %%a in (`pkg-config --variable=g_ir_scanner --dont-define-prefix gobject-introspection-1.0`) do if not exist "%%a" exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
